!*******************************************************************
!*                           PENEMF                                *
!*                                                                 *
!* Short description:                                              *
!*   Electric & magnetic fields and electrostatic potential (EMFs).*
!*                                                                 *
!*   Routines GETEMF() and GETSCP() are intended to be adapted by  *
!*   users to the required static fields. The code included below  *
!*   for these routines are given as an example: they define a     *
!*   constant EM field as introduced in the input file.            *
!*                                                                 *
!* Rev:                                                            *
!*   2024-02-04                                                    *
!*******************************************************************


!*******************************************************************
!*******************************************************************
!*    Modules section                                              *
!*******************************************************************
!*******************************************************************

      module EMFmod
!*******************************************************************
!*    Vars for the EMF routines.                                   *
!*******************************************************************
      implicit none
      save
      ! Constant electric & magnetic fields read from input file:
      logical :: isEMF,isELF
      real*8, dimension(3) :: efield0,bfield0
      real*8 uldv,ulde,ulem
      real*8, parameter :: EMFparamMin=1e-6,EMFparamMax=0.1
      end


      logical function isEMFf()
!*******************************************************************
!*    Returns true if EMFs are defined.                            *
!*    Used to access module vars without declaring its USE.        *
!*******************************************************************
      use EMFmod, only : isEMF
      implicit none

      isEMFf = .false.
      if (isEMF) isEMFf = .true.
      end


      logical function isELFf()
!*******************************************************************
!*    Returns true if en electric field is defined.                *
!*    Used to access module vars without declaring its USE.        *
!*******************************************************************
      use EMFmod, only : isELF
      implicit none

      isELFf = .false.
      if (isELF) isELFf = .true.
      end


!*******************************************************************
!*******************************************************************
!*    Routines section                                             *
!*******************************************************************
!*******************************************************************

      subroutine getemf(x,y,z,ex,ey,ez,bx,by,bz)
!*******************************************************************
!*    Returns the electric and magnetic field components at a point*
!*                                                                 *
!*    Input:                                                       *
!*      (x,y,z): Cartesian coordinates of the point (cm)           *
!*    Output:                                                      *
!*      (ex,ey,ez): Electric field components (volt/cm)            *
!*      (bx,by,bz): Magnetic field components (gauss)              *
!*    Comments:                                                    *
!*      - Defines a constant EMF.                                  *
!*      - Users should adapt this routine to return the actual EMF.*
!*      - PENELOPE does not check the consistency of the fields,   *
!*        ie, it does not check if rot E = div B = 0.              *
!*        It is up to the user to ensure these conditions are met  *
!*        to define proper static fields.                          *
!*      - 1 gauss = 1.0E-4 tesla                                   *
!*******************************************************************
      use EMFmod, only : efield0,bfield0
      implicit none
      real*8 x,y,z,ex,ey,ez,bx,by,bz

      ! Constant electric & magnetic fields as read from input file:
      ex = efield0(1)
      ey = efield0(2)
      ez = efield0(3)
      bx = bfield0(1)
      by = bfield0(2)
      bz = bfield0(3)
      end


      subroutine getscp(x,y,z,V)
!*******************************************************************
!*    Scalar EM potential                                          *
!*                                                                 *
!*    Input:                                                       *
!*      xyz: Cartesian coordinates of the point (cm)               *
!*    Output:                                                      *
!*      V: electrostatic potential (volt)                          *
!*    Comments:                                                    *
!*      - Defines a constant EMF.                                  *
!*      - Users should adapt this routine to return the actual EMF.*
!*******************************************************************
      use EMFmod, only : efield0
      implicit none
      real*8 x,y,z,V,r(3)

      r = (/x,y,z/)
      ! Electrostatic potential for the constant electric field:
      V = -dot_product(efield0,r)
      end


      subroutine iniEMF
!*******************************************************************
!*    Initializes. Reads data from input file.                     *
!*******************************************************************
      use EMFmod
      implicit none
      real*8 emax

      logical isVox
      character*(*) secid,eos
      parameter (secid=
     &  '[SECTION EMF v.2024-02-04]')
      parameter (eos='[END OF EMF SECTION]')
      character*80 buffer
      real*8, parameter :: zero=1.0d-32         ! Zero for EMFs^2

      write(*,*) ''
      write(*,'(a)')
     & '>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>'
      call getline(buffer,0)
      if (index(buffer,secid).eq.0) then
        write(*,'(a)') 'iniEMF:ERROR: incorrect section header;'
        write(*,'(a,a)') '  expecting to find: ',secid
        write(*,'(a,a)') '  found instead:     ',buffer
        stop
      endif
      write(*,'(a)') secid

      write(*,'(a)') 'Electric field (V/cm):'
      read(*,*) efield0                                 ! 3 Cartesian components
      write(*,'(3(1x,es12.5))') efield0
      write(*,'(a)') 'Magnetic field (G):'
      read(*,*) bfield0                                 ! 3 Cartesian components
      write(*,'(3(1x,es12.5))') bfield0

      isELF = .false.                                   ! Init electric field flag to false
      if (maxval(efield0**2).gt.zero) isELF = .true.    ! Set flag electric field present
      isEMF = .false.                                   ! Init EMF flag to false
      if (isELF.or.maxval(bfield0**2).gt.zero) then     ! Set flag EMF present
        isEMF = .true.
      else
        efield0 = 0.0                                   ! Set to zero in case user used a too small value
        bfield0 = 0.0
      endif

      if (isEMF) then                                   ! EMF present, read transport params
        write(*,'(a)')
     &    'Max fractional change in direction (param ULDV) set to:'
        read(*,*) uldv
        write(*,'(1x,es12.5)') uldv
        write(*,'(a)')
     &    'Max fractional change in energy (param ULDE) set to:'
        read(*,*) ulde
        write(*,'(1x,es12.5)') ulde
        write(*,'(a)')
     &    'Max fractional change in fields (param ULEM) set to:'
        read(*,*) ulem
        write(*,'(1x,es12.5)') ulem
        if (max(uldv,ulde,ulem).gt.EMFparamMax) then
          write(*,'(a,es12.5)')
     &      'iniEMF:ERROR: UL params exceed ',EMFparamMax
          stop
        endif
        if (min(uldv,ulde,ulem).lt.EMFparamMin) then
          write(*,'(a,es12.5)')
     &      'iniEMF:ERROR: UL params less than ',EMFparamMin
          stop
        endif
      else                                              ! EMF not present, skip transport params
        read(*,*) uldv                                  ! Read dummy lines
        read(*,*) uldv
        read(*,*) uldv
      endif

      if (isEMF) then
        write(*,'(a)') 'EMFs active'
      else
        write(*,'(a)') 'EMFs inactive'
      endif

      read(*,'(a80)') buffer
      if (index(buffer,eos).eq.0) then
        write(*,'(a)')
     &    'EDPinitally:ERROR: End-Of-Section mark not found'
        write(*,'(a,a)') '  expecting to find: ',eos
        write(*,'(a,a)') '  found instead:     ',buffer
        stop
      endif
      write(*,'(a)') '>>>> EMF initialization finished >>>>'
      end


!>>>> End Of File >>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>>
